/**
 * @file
 * @brief Tests to make sure the policy server can send chunked response data
 *
 * There are a few tests in here which try different methods of sending
 * chunked data back from the server to the redirector.  We need to
 * make sure all data segments are properly reassembled reglardless of
 * how they make it across the network.
 *
 * One test sends it byte by byte
 * Another test sends it chunk by chunk
 * Another test adds a delay between each byte
 *
 * Netsweeper Squid URL Redirector
 * This application is designed to work with the Netsweeper Internet Content Filtering System.
 *
 * Copyright 2010 Netsweeper Inc. All Rights Reserved.
 * Netsweeper, Netsweeper Inc. is either a copyrighted or registered trademark of Netsweeper Inc.
 */
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <pthread.h>
#include "jopen.h"

#include "CUnit/Basic.h"

#include "socketserver.h"

static void connect_callback(struct client_socket* s)
{
	printf("%p: Connected\n", s);
}
static void recv_callback(struct client_socket* s)
{
	int len;
	printf("%p: Data %s", s, s->rxbuffer);


	if ( strchr(s->rxbuffer, '\n') ) {
		len = 0;
		while ( s->rxlen - len > 0 ) {
			printf("sending %c\n", s->rxbuffer[len]);
			socketserver_send(s, s->rxbuffer+len, 1);
			len++;
		}
		s->rxlen=0;
	}
}
static void recv_random_callback(struct client_socket* s)
{
	int len;
	int tosend;
	printf("%p: Data %s", s, s->rxbuffer);


	if ( strchr(s->rxbuffer, '\n') ) {
		len = 0;
		while ( s->rxlen - len > 0 ) {
			tosend = (s->rxlen - len) / ((random() % 5) +1);
			printf("sending %d bytes %d bytes left\n", tosend, s->rxlen - (tosend + len));
			socketserver_send(s, s->rxbuffer+len, tosend);
			len+=tosend;
		}
		s->rxlen=0;
	}
}
static void recv_delay_callback(struct client_socket* s)
{
	int len;
	printf("%p: Data %s", s, s->rxbuffer);


	if ( strchr(s->rxbuffer, '\n') ) {
		len = 0;
		while ( s->rxlen - len > 0 ) {
			printf("sending %c\n", s->rxbuffer[len]);
			socketserver_send(s, s->rxbuffer+len, 1);
			len++;
			usleep(1);
		}
		s->rxlen=0;
	}
}
static void disconnect_callback(struct client_socket* s)
{
	printf("%p: Disconnect\n", s);
}

void chunked_squid25_simple_test()
{
	struct socketserver* svr;
	JFILE* fp;
	int i;
	char sendbuffer[256];
	char buffer[256];
	int ret;
	char* ptr;

	// Create a new amazing server
	svr = socketserver_create(3435, connect_callback, recv_callback, disconnect_callback);

	printf("%s(): Waiting for server startup\n", __FUNCTION__);

	fp = jopen("../redir localhost -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<100; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		ptr=strchr(sendbuffer, ' ');
		if ( ptr ) {
			*ptr='\0';
		}
		ptr=strchr(buffer, ' ');
		if ( ptr ) {
			*ptr='\0';
		}
		ptr=strchr(buffer, '\n');
		if ( ptr ) {
			*ptr='\0';
		}

		if ( strcmp(buffer, sendbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != sendbuffer %s\n", ret, buffer, sendbuffer);
			CU_FAIL("We did not get the response we expected");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	
	socketserver_signal_shutdown(svr);
	socketserver_waitfor_shutdown(svr);
	jclose(fp);

	CU_PASS("We have passed this test");

	return;
}

void chunked_squid25_random_test()
{
	struct socketserver* svr;
	JFILE* fp;
	int i;
	char sendbuffer[256];
	char buffer[256];
	int ret;
	char* ptr;

	// Create a new amazing server
	svr = socketserver_create(3435, connect_callback, recv_random_callback, disconnect_callback);

	printf("%s(): Waiting for server startup\n", __FUNCTION__);

	fp = jopen("../redir localhost -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<100; i++) {
		sprintf(sendbuffer, "http://google.ca/%d-%d-%d/test%d -/- - -\n", i, i, i, i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		ptr=strchr(sendbuffer, ' ');
		if ( ptr ) {
			*ptr='\0';
		}
		ptr=strchr(buffer, ' ');
		if ( ptr ) {
			*ptr='\0';
		}
		ptr=strchr(buffer, '\n');
		if ( ptr ) {
			*ptr='\0';
		}

		if ( strcmp(buffer, sendbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != sendbuffer %s\n", ret, buffer, sendbuffer);
			CU_FAIL("We did not get the response we expected");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	
	socketserver_signal_shutdown(svr);
	socketserver_waitfor_shutdown(svr);
	jclose(fp);

	CU_PASS("We have passed this test");

	return;
}


void chunked_squid25_delay_test()
{
	struct socketserver* svr;
	JFILE* fp;
	int i;
	char sendbuffer[256];
	char buffer[256];
	int ret;
	char* ptr;

	// Create a new amazing server
	svr = socketserver_create(3435, connect_callback, recv_delay_callback, disconnect_callback);

	printf("%s(): Waiting for server startup\n", __FUNCTION__);

	fp = jopen("../redir localhost -f http://failcloseurl");
	if ( ! fp ) {
		fprintf(stderr, "Could not open ../redir\n");
		socketserver_signal_shutdown(svr);
		socketserver_waitfor_shutdown(svr);
		return;
	}

	for(i=0; i<100; i++) {
		sprintf(sendbuffer, "http://google.ca/%d -/- - -\n", i);
		ret = write(fp->to_stdin, sendbuffer, strlen(sendbuffer));

		int len=0;
		buffer[0]='\0';
		while ( strchr(buffer, '\n') == NULL ) {
			ret=read(fp->from_stdout, buffer+len, 1024);
			if ( ret > 0 ) {
				len+=ret;
				buffer[len]='\0';
			}
		}

		ptr=strchr(sendbuffer, ' ');
		if ( ptr ) {
			*ptr='\0';
		}
		ptr=strchr(buffer, ' ');
		if ( ptr ) {
			*ptr='\0';
		}
		ptr=strchr(buffer, '\n');
		if ( ptr ) {
			*ptr='\0';
		}

		if ( strcmp(buffer, sendbuffer) ) {
			fprintf(stderr, "ret = %d buffer %s != sendbuffer %s\n", ret, buffer, sendbuffer);
			CU_FAIL("We did not get the response we expected");
			socketserver_signal_shutdown(svr);
			socketserver_waitfor_shutdown(svr);
			jclose(fp);
			return;
		}
	}
	
	socketserver_signal_shutdown(svr);
	socketserver_waitfor_shutdown(svr);
	jclose(fp);

	CU_PASS("We have passed this test");

	return;
}

int chunked_squid25_test_init()
{
	CU_pSuite pSuite = NULL;

	// This is our test suite for test1
	pSuite = CU_add_suite("Chunked Squid 2.5 Tests", NULL, NULL);
	if (NULL == pSuite) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	if ( (NULL == CU_add_test(pSuite, "Chunked Squid 2.5 by Byte", chunked_squid25_simple_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	if ( (NULL == CU_add_test(pSuite, "Chunked Random Squid 2.5", chunked_squid25_random_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	if ( (NULL == CU_add_test(pSuite, "Chunked Squid 2.5 With Delay", chunked_squid25_delay_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
	}
	return 0;
}
